#version 300 es

uniform highp mat4 _ViewMatrix;
uniform highp mat4 _ViewProjectionMatrix;
uniform highp mat4 boneMatrices[8];
uniform highp mat4 modelMatrix;

layout (std140) uniform InstanceBuffer
{
    highp mat4 instanceModelMatrices[ 256 ];
};

layout (location = 0) in highp vec4 aPosition;
layout (location = 2) in highp vec3 aNormal;
layout (location = 4) in highp vec4 aBoneIndex;
layout (location = 5) in highp vec4 aBoneWeights;

out highp vec4 vPosition;
out highp vec3 vNormal;

void main()
{
    highp vec4 position = aPosition;
#ifdef SKIN
    highp ivec4 boneIndex = ivec4( aBoneIndex );
    
    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;

    gl_Position = _ViewProjectionMatrix * modelMatrix * position;
    highp mat4 modelViewMatrix = _ViewMatrix * modelMatrix;
#else
    gl_Position = _ViewProjectionMatrix * instanceModelMatrices[ gl_InstanceID ] * position;
    highp mat4 modelViewMatrix = _ViewMatrix * instanceModelMatrices[ gl_InstanceID ];
#endif
    vPosition = modelViewMatrix * position;
    vNormal = (modelViewMatrix * vec4( aNormal, 0.0 )).xyz;
}
