#version 300 es

uniform highp mat4 _ViewProjectionMatrix;
uniform highp mat4 modelMatrix;
uniform highp mat4 boneMatrices[8];
uniform highp vec4 textureMap_ST;
uniform highp vec3 _CameraPosition;

layout (location = 0) in highp vec4 aPosition;
layout (location = 1) in mediump vec2 aTexCoord;
layout (location = 2) in mediump vec3 aNormal;
layout (location = 4) in highp vec4 aBoneIndex;
layout (location = 5) in highp vec4 aBoneWeights;

out mediump vec3 vEnvTexCoord;

void main()
{
    highp vec4 position = aPosition;
#ifdef SKIN
    highp ivec4 boneIndex = ivec4( aBoneIndex );

    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;
#endif

    gl_Position = _ViewProjectionMatrix * modelMatrix * position;

    // http://www.ozone3d.net/tutorials/glsl_texturing_p04.php#part_43)
    highp vec4 worldPos    = modelMatrix * position;
    highp vec4 worldNormal = normalize( modelMatrix * vec4( aNormal, 0.0 ) );
    highp vec4 worldEye    = normalize( worldPos - vec4( _CameraPosition, 1.0 ) );
    
    vEnvTexCoord = reflect( worldEye.xyz, worldNormal.xyz );
}
