#version 300 es

uniform sampler2D depthNormalMap;
uniform sampler2D noiseMap;
uniform highp mat4 _ProjectionMatrix;
uniform highp int uKernelSize;
const int MAX_KERNEL_SIZE = 128;
uniform highp vec3 uKernelOffsets[ MAX_KERNEL_SIZE ];
uniform highp float uRadius;
uniform highp float uPower;

in mediump vec2 vTexCoord;
in highp vec3 vViewDir;

out lowp vec4 fragColor;

highp float ssao( in highp mat3 kernelBasis, in highp vec3 originPos, in highp float radius )
{
    highp float occlusion = 0.0;
	  
    for (highp int i = 0; i < uKernelSize; ++i)
    {
        highp vec3 samplePos = kernelBasis * uKernelOffsets[i];
        samplePos = samplePos * radius + originPos;
		
        // project sample position:
        highp vec4 offset = _ProjectionMatrix * vec4( samplePos, 1.0 );
        offset.xy /= offset.w; // only need xy
        offset.xy = offset.xy * 0.5 + 0.5; // scale/bias to texcoords
		
        // get sample depth:
        highp float sampleDepth = texture( depthNormalMap, offset.xy ).r;
		
        highp float rangeCheck = smoothstep( 0.0, 1.0, radius / abs( originPos.z - sampleDepth ) );
        occlusion += rangeCheck * step( sampleDepth, samplePos.z );
    }
	
    occlusion = 1.0 - (occlusion / float( uKernelSize ));
    return pow( abs( occlusion ), uPower );
}

void main()
{
    highp vec2 noiseTexCoords = vec2( textureSize( depthNormalMap, 0 )) / vec2( textureSize( noiseMap, 0 ) );
    noiseTexCoords *= vTexCoord;
	
    // get view space origin:
    highp float originDepth = texture( depthNormalMap, vTexCoord ).r;
    highp vec3 originPos = vViewDir * originDepth;

    // get view space normal:
    highp vec3 normal = -normalize( texture( depthNormalMap, vTexCoord ).gba );
		
    // construct kernel basis matrix:
    highp vec3 rvec = normalize( texture( noiseMap, noiseTexCoords ).rgb ) * 2.0 - 1.0;
    highp vec3 tangent = normalize( rvec - normal * dot( rvec, normal ) );
    highp vec3 bitangent = cross( tangent, normal );
    highp mat3 kernelBasis = mat3( tangent, bitangent, normal );
	
    fragColor = vec4( ssao( kernelBasis, originPos, uRadius ) );
}

