#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

uniform mat4 _ViewMatrix;
uniform mat4 _ViewProjectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 boneMatrices[8];

layout (std140) uniform InstanceBuffer
{
    mat4 instanceModelMatrices[ 1024 ];
};

layout (location = 0) in vec4 aPosition;
layout (location = 2) in vec3 aNormal;
layout (location = 4) in vec4 aBoneIndex;
layout (location = 5) in vec4 aBoneWeights;
layout (location = 9) in mat4 aInstanceModelMatrix;

out vec4 vPosition;
#ifdef DEPTH
out vec3 vNormal;
#endif

void main()
{
    vec4 position = aPosition;
#ifdef SKIN
    ivec4 boneIndex = ivec4( aBoneIndex );
    
    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;
    gl_Position = _ViewProjectionMatrix * modelMatrix * position;
    mat4 modelViewMatrix = _ViewMatrix * modelMatrix;
#else
    gl_Position = _ViewProjectionMatrix * instanceModelMatrices[ gl_InstanceID ] * position;
    mat4 modelViewMatrix = _ViewMatrix * instanceModelMatrices[ gl_InstanceID ];
#endif

    vPosition = modelViewMatrix * position;
#ifdef DEPTH
    vNormal = mat3( modelViewMatrix ) * aNormal;
#endif
}
