#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

uniform mat4 _ViewProjectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 _ViewMatrix;
uniform mat4 boneMatrices[8];
uniform vec4 textureMap_ST;
uniform vec3 _CameraPosition;

layout (location = 0) in vec4 aPosition;
layout (location = 1) in vec2 aTexCoord;
layout (location = 2) in vec3 aNormal;
layout (location = 4) in vec4 aBoneIndex;
layout (location = 5) in vec4 aBoneWeights;

out vec3 vEnvTexCoord;

void main()
{
    vec4 position = aPosition;
#ifdef SKIN
    ivec4 boneIndex = ivec4( aBoneIndex );

    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;
#endif
    vec4 worldPos = modelMatrix * position;
    
    gl_Position = _ViewProjectionMatrix * worldPos;
    
    // http://www.ozone3d.net/tutorials/glsl_texturing_p04.php#part_43)
    vec4 worldNormal = normalize( modelMatrix * vec4( aNormal, 0.0 ) );
    vec4 worldEye    = normalize( worldPos - vec4( _CameraPosition, 1.0 ) );

    vEnvTexCoord = -reflect( worldEye.xyz, worldNormal.xyz );
}
