#version 430 core

uniform mat4 _ViewProjectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 _ViewMatrix;
uniform mat4 boneMatrices[8];
uniform vec4 diffuseMap_ST;
uniform vec3 _LightDirection;
uniform vec3 _CameraPosition;
uniform mat4 _TPMatrix;

layout (location = 0) in vec4 aPosition;
layout (location = 1) in vec2 aTexCoord;
layout (location = 2) in vec3 aNormal;
layout (location = 3) in vec4 aTangent;
layout (location = 4) in vec4 aBoneIndex;
layout (location = 5) in vec4 aBoneWeights;

out vec2 vTexCoord;
out vec3 vViewVecTS;
out vec3 vLightVecTS;
#ifdef ENVIRONMENT_MAP
out vec3 vEnvTexCoord;
#endif
out vec3 vNormalWS;
out vec3 vTangentWS;
out vec3 vBitangentWS;

// Shadow stuff
out vec4 vPositionWS;
out vec4 vProjCoord;

void main()
{
    vec4 position = aPosition;
#ifdef SKIN
    ivec4 boneIndex = ivec4( aBoneIndex );

    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;
#endif
    vec4 worldPos = modelMatrix * position;

    gl_Position = _ViewProjectionMatrix * worldPos;

    mat4 modelViewMatrix = _ViewMatrix * modelMatrix;
    vec4 positionVS = modelViewMatrix * position;

    mat3 normalMatrix = mat3( modelViewMatrix );
    
    vec3 t = normalize( normalMatrix * aTangent.xyz );
    vec3 b = normalize( normalMatrix * (cross( aNormal, aTangent.xyz ) * aTangent.w) );
    vec3 n = normalize( normalMatrix * aNormal );
    mat3 tbn = transpose( mat3( t, b, n ) );

    vLightVecTS = (_ViewMatrix * vec4( -_LightDirection, 0.0f )).xyz;
    vLightVecTS = tbn * vLightVecTS;

    vViewVecTS = -positionVS.xyz;
    vViewVecTS = tbn * vViewVecTS;
    
    vTexCoord = aTexCoord * diffuseMap_ST.xy + diffuseMap_ST.zw;
    
    // Shadow stuff
    vPositionWS  = worldPos;
    vProjCoord = _TPMatrix * position;

    vNormalWS = normalize( modelMatrix * vec4( aNormal, 0.0f ) ).xyz;
    vTangentWS = normalize( modelMatrix * vec4( aTangent.xyz, 0.0f ) ).xyz;
    vec3 bitangentOS = cross( aNormal, aTangent.xyz ) * aTangent.w;
    vBitangentWS = normalize( modelMatrix * vec4( bitangentOS, 0.0f ) ).xyz;

#ifdef ENVIRONMENT_MAP
    vec4 worldNormal = normalize( modelMatrix * vec4( aNormal, 0.0f ) );
    vec4 worldEye    = normalize( worldPos - vec4( _CameraPosition, 1.0f ) );
    
    vEnvTexCoord = reflect( -worldEye.xyz, -worldNormal.xyz );
#endif
}
