cbuffer buffer : register(b0)
{
    column_major matrix _ViewProjectionMatrix;
    column_major matrix modelMatrix;
    float3 texelOffset;
    float4 textureMap_ST;
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex0 : TEXCOORD0;
    float2 Tex1 : TEXCOORD1;
    float2 Tex2 : TEXCOORD2;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 position = float4( input.Pos, 1 );
    position.y = -position.y; // Render-to-Texture produces flipped image.

    output.Pos = mul( _ViewProjectionMatrix, mul( modelMatrix, position ) );
    output.Tex0 = input.Tex - texelOffset.xy;
    output.Tex1 = input.Tex;
    output.Tex2 = input.Tex + texelOffset.xy;
    return output;
}

Texture2D textureMap : register(t0);
SamplerState SampleType : register(s0);

float4 PS( PS_INPUT input ) : SV_Target
{
    float3 color = textureMap.Sample( SampleType, input.Tex0 ).rgb * 0.33333;
    color += textureMap.Sample( SampleType, input.Tex1 ).rgb * 0.33333;
    color += textureMap.Sample( SampleType, input.Tex2 ).rgb * 0.33333;
    return float4( saturate(color), 1.0 );
}