cbuffer buffer : register(b0)
{
    float4 textureMap_ST;
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex : TEXCOORD0;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 position = float4( input.Pos, 1.0f );
    position.y = -position.y; // Render-to-Texture produces flipped image.

    output.Pos = position;
    output.Tex = input.Tex;
    return output;
}

Texture2D textureMap : register(t0);
SamplerState SampleType : register(s0);

float4 PS( PS_INPUT input ) : SV_Target
{
    int w, h;
    textureMap.GetDimensions( w, h );
    float2 texelSize = float2( 1.0f / float( w ), 1.0f / float( h ) );

    float4 fragColor = float4( 0.0f, 0.0f, 0.0f, 0.0f );
    int uBlurSize = 4; // Should equal to noise texture size.
    float2 hlim = float( -uBlurSize ) * 0.5f + 0.5f;

    for (int x = 0; x < uBlurSize; ++x)
    {
        for (int y = 0; y < uBlurSize; ++y)
        {
            float2 offset = float2( float( x ), float( y ) );
            offset += hlim;
            offset *= texelSize;

            fragColor += textureMap.Sample( SampleType, input.Tex + offset );
        }
    }

    return fragColor / float(uBlurSize * uBlurSize);
}