cbuffer buffer : register(b0)
{
    column_major matrix _ViewProjectionMatrix;
    column_major matrix modelMatrix;
#ifdef SKIN
    column_major matrix boneMatrices[8];
#endif
}

cbuffer InstanceBuffer : register(b1)
{
    column_major matrix instanceModelMatrices[ 1024 ];
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
#ifdef SKIN
    float4 BoneInd : BONE_INDICES;
    float4 Weights : BONE_WEIGHTS;
#endif
    uint InstanceId : SV_InstanceID;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 position = float4( input.Pos, 1.0f );
#ifdef SKIN
    int4 boneIndex = int4( input.BoneInd );
    
    position  = mul( boneMatrices[ boneIndex.x ], float4(input.Pos, 1.0f) * input.Weights.x );
    position += mul( boneMatrices[ boneIndex.y ], float4(input.Pos, 1.0f) * input.Weights.y );
    position += mul( boneMatrices[ boneIndex.z ], float4(input.Pos, 1.0f) * input.Weights.z );
    position += mul( boneMatrices[ boneIndex.w ], float4(input.Pos, 1.0f) * input.Weights.w );

    output.Pos = mul( _ViewProjectionMatrix, mul( modelMatrix, position ) );
#else
    output.Pos = mul( _ViewProjectionMatrix, mul( instanceModelMatrices[ input.InstanceId ], position ) );
#endif

    output.Pos.z = output.Pos.z * 0.5 + 0.5; // OpenGL->D3D11 unit cube fixup.
    return output;
}

float4 PS( PS_INPUT input ) : SV_Target
{
    float linearDepth = input.Pos.z;

    // Compute partial derivatives of depth. 
    float dx = ddx( linearDepth );
    float dy = ddy( linearDepth );

    float moment1 = linearDepth;
    // Compute second moment over the pixel extents.  
    float moment2 = linearDepth * linearDepth + 0.25f * (dx * dx + dy * dy);
    return float4( moment1, moment2, 0.0f, 1.0f );
}