cbuffer buffer : register(b0)
{
    float4 textureMap_ST;
    float bloomFactor;
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex : TEXCOORD0;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 position = float4( input.Pos, 1.0f );
    position.y = -position.y; // Render-to-Texture produces flipped image.

    output.Pos = position;
    output.Tex = input.Tex;
    return output;
}

Texture2D textureMap : register(t0);
Texture2D bloomMap : register(t1);
Texture2D ssaoMap : register(t2);
SamplerState textureSampler : register(s0);
SamplerState bloomSampler : register(s1);
SamplerState ssaoSampler : register(s2);

float4 PS( PS_INPUT input ) : SV_Target
{
    float exposure = 2.5f;
    float brightMax = 2.0f;

    float4 color = textureMap.Sample( textureSampler, input.Tex );
    float4 ssao = ssaoMap.Sample( ssaoSampler, float2(input.Tex.x, 1.0f - input.Tex.y) );

    color *= ssao.r;

    float4 colorBloom = bloomMap.Sample( bloomSampler, input.Tex );

    // Add bloom to the image
    color += colorBloom * bloomFactor;

    // Perform tone-mapping
    float YD = exposure * (exposure / brightMax + 1.0f) / (exposure + 1.0f);
    color *= YD;

    return color;
}
