#ifndef LIGHT_H
#define LIGHT_H

#ifndef SCENEGRAPHNODE_H
#    include "SceneGraphNode.hpp"
#endif

struct Vec3;

/// Light.
class Light : public SceneGraphNode
{
  public:
    /** Light's type. */
    enum class Type
    {
        Directional, /** A light placed infinitely far away.  It affects everything in the scene. */
        Point, /** A light with a punctual source.  Affects every model in its radius. */
        Spot /** A light with conical area.  Affects every model in its cone. */
    };
    
    /** Destructor. */
    virtual ~Light() {}

    /** \return type. */
    virtual Type GetLightType() const = 0;

    /**
       \param enable True, if the light casts shadow.  Disabled by default.  Shadows are currently only supported for directional lights.
       \param shadowMapSize Shadow map's size in pixels.  Width and height will be set to shadowMapSize.
     */
    virtual void SetCastShadow( bool enable, int shadowMapSize ) = 0;
    
    /** \param newColor Color.  Values are in range 0-1. */
    virtual void SetColor( const Vec3& newColor ) = 0;
    
    /** \return Color in range 0-1. */
    virtual Vec3 GetColor() const = 0;

    /** \param degrees Spot light's cone angle in degrees. */
    virtual void SetConeAngle( float degrees ) = 0;

    /** \param newDirection Normalized direction vector.  Doesn't affect point lights. */
    virtual void SetDirection( const Vec3& newDirection ) = 0;

    /** \param position Position. */
    virtual void SetPosition( const Vec3& position ) = 0;

    /** \param radius Radius.  Not affected by scene graph. */
    virtual void SetRadius( float radius ) = 0;
};

#endif
