#include "SceneGraphNode.hpp"
#include <iostream>
#include "Matrix.hpp"
#include "Scene.hpp"

Scene* SceneGraphNode::sScene = nullptr;

void SceneGraphNode::RegisterScene( Scene* scene )
{
    sScene = scene;
}

void SceneGraphNode::UnregisterScene()
{
    sScene = nullptr;
}

SceneGraphNode::SceneGraphNode()
  : type( SceneGraphNode::Type::None )
  , scale( 1 )
  , parent( nullptr )
  , layerMask ( 0x1 )
{
}

SceneGraphNode::~SceneGraphNode()
{
}

bool SceneGraphNode::AddChild( SceneGraphNode* child )
{
    if (!child)
    {
        return false;
    }

    bool childFound = false;
    Root()->FindChild( child, childFound );

    if (childFound)
    {
        return false;
    }

    child->parent = this;
    children.push_front( child );
    
    // FIXME: This adds the node to all scenes.
    if (sScene)
    {
        sScene->OnNodeAdded( child );
    }

    return true;
}

bool SceneGraphNode::ContainsChild( const SceneGraphNode* node ) const
{
    bool nodeFound = false;
    FindChild( node, nodeFound );
    return nodeFound;
}

SceneGraphNode* SceneGraphNode::Root()
{
    SceneGraphNode* node = this;

    while (node->Parent())
    {
        node = node->Parent();
    }

    return node;
}

void SceneGraphNode::CleanDirties()
{
    dirtyTransform = false;
    
    for (auto& child : children)
    {
        child->CleanDirties();
    }
}

void SceneGraphNode::FindChild( const SceneGraphNode* node, bool& outFound ) const
{
    if (this == node)
    {
        outFound = true;
    }

    for (const auto& child : children)
    {
        child->FindChild( node, outFound );       
    }
}

void SceneGraphNode::FindFirstChildWithName( const char* aName, SceneGraphNode** outNode )
{
    if (GetName() == aName)
    {
        *outNode = this;
        return;
    }
    
    for (const auto& child : children)
    {
        child->FindFirstChildWithName( aName, outNode );
    }
}

void SceneGraphNode::SolveMatrices()
{
    if (IsDirty())
    {
        rotation.GetMatrix( modelMatrix );
        modelMatrix.Scale( scale, scale, scale );
        modelMatrix.Translate( position );
        
        if (parent)
        {
            Matrix44::Multiply( modelMatrix, parent->modelMatrix, modelMatrix );
        }
    }
    
    for (auto& child : children)
    {
        child->SolveMatrices();
    }
}
