#include "RenderQueue.hpp"
#include "Renderer.hpp"
#include "TextureImpl.hpp"
#include <cassert>

RenderQueue::Vec3uv::Vec3uv() : u( 0 ), v( 0 ) {}

RenderQueue::Vec3uv::Vec3uv( float aX, float aY, float aZ, float aU, float aV ) : x( aX ), y( aY ), z( aZ ), u( aU ), v( aV ) {}

struct RenderQueue::Impl
{
    Impl() : lastEntry( -1 ) {}
    void Draw();
    
    std::vector< RenderQueueEntry > entries;
    std::vector< unsigned int > VBOids;
    int lastEntry;
};

RenderQueue::RenderQueue( Shader& ) : m( new Impl ) {}

RenderQueue::~RenderQueue() {}

void RenderQueue::Add( PrimitiveType aPrimitive, const std::vector< Vec3uv >& aVertices,
                       const TextureImpl* aTexture,
                       int aSortingValue, const Vec4& color )
{
    // Finds an entry that has same state as the arguments and inserts vertices
    // there so we can render them using one draw call.
    for (auto& i : m->entries)
    {
        if (i.sortingValue == aSortingValue &&
            ((i.texture && aTexture) ? (i.texture->Id() == aTexture->Id()) : &i.texture == &aTexture) &&
            i.red == color.x && i.green == color.y && i.blue == color.z && i.opacity == color.w)
        {
            i.vertices.insert( std::end( i.vertices ), std::begin( aVertices ), std::end( aVertices ) );
            return;
        }
    }
    
    // Creates a new entry.
    ++m->lastEntry;
    
    if ((std::size_t)m->lastEntry == m->entries.size())
    {
        const std::size_t newSize = m->entries.size() == 0 ? 20 : m->entries.size() * 2;
        
        m->entries.resize( newSize );
        m->VBOids.resize( newSize );
    }
    
    RenderQueueEntry e( aSortingValue, aTexture, aVertices, color.x, color.y, color.z, color.w, aPrimitive );
    m->entries[ m->lastEntry ] = e;
}

void RenderQueue::Flush( float, float, const Matrix44& )
{
    m->Draw();
    m->entries.clear();
    m->lastEntry = -1;
}

void RenderQueue::Deinit() const
{
}

void RenderQueue::Impl::Draw()
{
}
