#ifndef RENDERER_NULL_H
#define RENDERER_NULL_H

#ifndef RENDERER_H
#    include "Renderer.hpp"
#endif
#ifndef INCLUDE_MEMORY
#    include <memory>
#    define INCLUDE_MEMORY
#endif
#include <string>

struct Vec4;

class RendererNULL : public Renderer
{
  public:
    RendererNULL();
    virtual ~RendererNULL() {}
    virtual void AddShaderSearchPath( const char* path ) const;
    virtual void ClearScreen() const;
    virtual void Deinit();
    virtual void Draw( const Texture* texture, int x, int y, const Vec4& color );
    virtual void Draw( const Texture* texture, int x, int y, int width, int height, const Vec4& color );
    virtual void Draw( const char* text, const char* fontName, float scale, int x, int y, const Vec4& color );
    virtual void Draw( Scene* scene );
    virtual void DrawStatistics();
    virtual void FinishFrame();
    virtual double GetTime() const;
    virtual int Height() const;
    virtual Error::Enum LoadFont( const char* fontName, const char* fontBitmapPath, const char* fontMetadataPath );
    virtual Error::Enum OpenWindow( int width, int height, bool fullScreen, int antialiasSamples, bool createWindowAndContext );
    virtual void Resize( int width, int height );
    virtual void SetBloom( bool enabled, float factor, int blurIterations );
    virtual void SetClearColor( float red, float green, float blue );
    virtual void SetWindowTitle( const char* title ) const;
    virtual void SetSSAO( bool enable, float radius, float power );
    virtual void SwapBuffers();
    virtual int Width() const;
    virtual void BeginQuery();
    virtual double EndQuery();

    void SetBuiltinAssetPath( const std::string& path );

  private:
    struct Impl;
    std::unique_ptr< Impl > m;
};

#endif
