#ifndef FONTER_H
#define FONTER_H

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif
#ifndef INCLUDE_VECTOR_H
#    include <vector>
#    define INCLUDE_VECTOR_H
#endif
#ifndef ERRORS_H
#    include "Errors.hpp"
#endif
#ifndef RENDERQUEUE_H
#    include "RenderQueue.hpp"
#endif

class TextureImpl;

/**
   Fonter loads bitmap fonts and handles their data.
   Currently supports fonts generated by BMFont.
   Limitation: Only supports 8-bit text encoding.
   Note: Only tested with monospace fonts.
 */
class Fonter
{
public:
    Fonter();
    Fonter( const Fonter& fonter );
    Fonter& operator=( const Fonter& fonter );
    ~Fonter();

    std::vector< RenderQueue::Vec3uv > CreateGeometry( const char* text, float x, float y, float scale ) const;
    Error::Enum Load( const char* fontBitmapPath, const char* fontMetadataPath );
    int TextWidth( const char* text ) const;
    const TextureImpl* Texture() const;

private:
    struct Impl;
    std::unique_ptr< Impl > m;
};
#endif
