#ifndef FORWARD_PLUS_H
#define FORWARD_PLUS_H

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif
#ifndef PLATFORM_H
#    include "Platform.hpp"
#endif

class ComputeShader;
struct RendererState;
class Shader;
class TextureImpl;
struct Vec3;
struct Vec4;

// these are half-precision (i.e. 16-bit) float values, 
// stored as unsigned shorts
struct SpotParams
{
    unsigned short lightDirX;
    unsigned short lightDirY;
    unsigned short cosineOfConeAngleAndLightDirZSign;
    unsigned short falloffRadius;
};

/**
  Forward+ light culler.  Currently only supports point lights and implemented in compute shader.
*/
struct ForwardPlus
{
    ForwardPlus();
    virtual ~ForwardPlus();
    void Init();
    void Deinit();
    unsigned GetMaxNumLightsPerTile() const;
    int GetNextPointLightBufferIndex();
    int GetNextSpotLightBufferIndex();
    unsigned GetNumTilesX() const { return (unsigned)((Screen::Width() + TILE_RES - 1) / (float)TILE_RES); }
    unsigned GetNumTilesY() const { return (unsigned)((Screen::Height() + TILE_RES - 1) / (float)TILE_RES); }
    void CullLights( const RendererState& state, ComputeShader& cullShader, TextureImpl& depthNormalTarget );
    void SetUniformsForShading( Shader& shader );
    void SetPointLightColor( int bufferIndex, const Vec3& color );
    void SetSpotLightColor( int bufferIndex, const Vec3& color );
    void SetPointLightPositionAndRadius( int bufferIndex, const Vec4& positionAndRadius );
    void SetSpotLightPositionAndRadius( int bufferIndex, const Vec4& positionAndRadius );
    void SetSpotLightParams( int bufferIndex, const Vec3& direction, float coneAngleCosRad, float falloff );

private:
    const int TILE_RES = 16;

    struct Impl;
    std::unique_ptr< Impl > m;
};

#endif
