#ifndef GFX_DEVICE_HPP
#define GFX_DEVICE_HPP

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif

class TextureImpl;

enum class DepthFunc
{
    None,
    EqualWriteOn,
    LessWriteOn,
    LessOrEqualWriteOn,
    EqualWriteOff,
    LessWriteOff,
    LessOrEqualWriteOff
};

enum class BlendMode
{
    AlphaBlend,
    Additive,
    Off
};

enum class RasterizerState
{
    CullBackCCW,
    CullNoneCW,
    CullNoneCCW
};

/**
   Sets graphics state such as blending modes, culling and clear color.
 */
class GfxDevice
{
  public:
    static GfxDevice& Instance();
    GfxDevice();
    ~GfxDevice();
    /** Must be called after D3D11 device is created.  Currently does nothing on GL. */
    void Init();
    void Deinit();
    void ClearScreen();
    void SetBlendMode( BlendMode blendMode );
    void SetClearColor( float red, float green, float blue );
    void SetDepthFunc( DepthFunc test );
    void SetRasterizerState( RasterizerState rasterizerState );
    void SetViewport( int left, int right, int top, int bottom );
    // FIXME: sRGB write should probably be part of render texture properties and set when enabling target.
    void Set_sRGB_Writes( bool enable );
    // Doesn't do anything on GLES3.
    void SetWireframe( bool enable );
    
    /** \return Target bound by last call to SetRenderTarget() */
    const TextureImpl* CurrentRenderTarget();

    /**
     \param Target target texture.  Use nullptr for backbuffer screen target.  Must be generated with CreateRenderTexture() or CreateRenderCubemap().
     \param cubeMapFace Cube map face index for cube map texture [0-5].  Indexed by GL_TEXTURE_CUBE_MAP_*.
     */
    void SetRenderTarget( const TextureImpl* target, unsigned cubeMapFace = 0 );

  private:
    struct Impl;
    std::unique_ptr< Impl > m;
};
#endif
