#ifndef MATERIAL_IMPL_H
#define MATERIAL_IMPL_H

#ifndef MATERIAL_H
#    include "Material.hpp"
#endif
#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif

class Shader;
struct Matrix44;
class Texture;
struct Vec3;
struct Vec4;

class MaterialImpl : public Material
{
  public:
    static void AddShaderSearchPath( const std::string& path );
    static MaterialImpl& Default();
    static void SetDefaultShader( Shader& shader );

    MaterialImpl();
    virtual ~MaterialImpl();
    
    void LoadFromFile( const char* path ) override;
    const char* Name() const override;
    bool Opaque() const override;
    void SetOpaque( bool opaque ) override;
    Error::Enum SetShaders( const std::string& vertexShaderPath,
                                    const std::string& fragmentShaderPath,
                                    const std::string& geometryShaderPath,
                                    const std::string& defines ) override;
    void SetFloat( const std::string& name, float value ) override;
    void SetMatrix( const std::string& name, const Matrix44& matrix ) override;
    void SetName( const char* name ) override;
    void SetTexture( const std::string& name, const Texture* texture ) override;
    void SetVector( const std::string& name, const Vec3& value ) override;
    void SetVector( const std::string& name, const Vec4& value ) override;
    /// Sends the variables to shader and activates it.
    void Apply();
    Shader& GetShader();
    const char* Path() const;

  private:
    struct Impl;
    std::unique_ptr< Impl > m;
};

#endif
