#ifndef MODEL_IMPL_H
#define MODEL_IMPL_H

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif
#ifndef INCLUDE_VECTOR_H
#    include <vector>
#    define INCLUDE_VECTOR_H
#endif
#ifndef ERRORS_H
#    include "Errors.hpp"
#endif
#ifndef MODEL_H
#    include "Model.hpp"
#endif

struct Matrix44;
class Mesh;
class Quaternion;
class VertexBuffer;
struct Vec3;

/// Model's implementation.
class ModelImpl : public Model
{
  public:
    /**
     Reloads a model if exists.

     \param path Path.
     */
    static void Reload( const std::string& path );
    static void Deinit();

    ModelImpl();
    ModelImpl( const ModelImpl& other );
    ModelImpl& operator=( const ModelImpl& other );
    virtual ~ModelImpl();
    void GetAABB( Vec3& outMin, Vec3& outMax ) const override;
    std::vector< std::string > GetMeshNames() const override;
    std::vector< std::string > GetMeshJointNames( const std::string& meshName ) const override;
    const std::string& GetPath() const override;
    void ApplyAnimation( int firstKeyFrame, int lastKeyFrame ) override;
    Material* GetMeshMaterial( const char* meshName ) override;
    void EnableWireframe( bool enable ) override;
    bool IsWireframe() const;
    void SetMaterialForMesh( Material* material, const std::string& meshName ) override;
    void SetMaterialForMeshSubstring( Material* material, const std::string& meshNameSubstring ) override;
    const VertexBuffer& CombinedOpaqueMeshes() const;
    /// \return AABB max in model space.
    const Vec3& AABBMax() const;
    /// \return AABB min in model space.
    const Vec3& AABBMin() const;
    /// \return AABB max in world space.
    const Vec3& AABBMaxWorld() const;
    /// \return AABB min in world space.
    const Vec3& AABBMinWorld() const;
    Error::Enum LoadFromFile( const std::string& path );
    std::vector< Mesh >& Meshes();
    const Matrix44& ModelMatrix() const;
    void SolveWorldAABB();

  private:
    struct Impl;
    std::unique_ptr< Impl > m;
};

#endif
