#include "MouseInput.hpp"
// for memcpy inside SDL
#include <string.h>
#include <SDL/SDL.h>

#ifdef AETHER3D_WINDOWS
#define AETHER3D_API __declspec(dllexport)
#else
#define AETHER3D_API 
#endif

extern SDL_Window* mainWindow;

extern "C" AETHER3D_API MouseInput* CreateMouseInput()
{
    return new MouseInput();
}

void MouseInput::GetPosition( int& outX, int& outY ) const
{
    if (SDL_GetRelativeMouseMode() == SDL_TRUE)
    {
        SDL_GetRelativeMouseState( &outX, &outY );
    }
    else
    {
        SDL_GetMouseState( &outX, &outY );
    }
}

void MouseInput::SetPosition( int x, int y ) const
{
    SDL_WarpMouseInWindow( mainWindow, x, y );
}

void MouseInput::SetRelative( bool enable ) const
{
    SDL_SetRelativeMouseMode( enable ? SDL_TRUE : SDL_FALSE );
}

void MouseInput::ShowCursor( bool enable ) const
{
    SDL_ShowCursor( enable ? 1 : 0 );
}

bool MouseInput::IsDown( ae3d::mouseinput::Enum aButton ) const
{
    Uint32 buttonMask = SDL_GetMouseState( nullptr, nullptr );
    
    if (buttonMask & SDL_BUTTON( 1 ) && aButton == ae3d::mouseinput::MOUSE_BUTTON_LEFT)
    {
        return true;
    }
    else if (buttonMask & SDL_BUTTON( 2 ) && aButton == ae3d::mouseinput::MOUSE_BUTTON_MIDDLE)
    {
        return true;
    }
    else if (buttonMask & SDL_BUTTON( 3 ) && aButton == ae3d::mouseinput::MOUSE_BUTTON_RIGHT)
    {
        return true;
    }
    
    return false;
}
