#import "ViewController.h"
#include "TextureImpl.hpp"
#include "RendererImpl.hpp"

@interface ViewController () {
    TextureImpl textureFromAtlas1;
    TextureImpl* texture;
}
@property (strong, nonatomic) EAGLContext *context;

- (void)setupGL;
- (void)tearDownGL;

@end

@implementation ViewController

GLKView *globalView = NULL;

void BindDrawable()
{
    [globalView bindDrawable];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES3];

    if (!self.context) {
        NSLog(@"Failed to create ES context");
    }
    
    GLKView *view = (GLKView *)self.view;
    view.context = self.context;
    view.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    globalView = view;

    [self setupGL];
}

- (void)dealloc
{    
    [self tearDownGL];
    
    if ([EAGLContext currentContext] == self.context) {
        [EAGLContext setCurrentContext:nil];
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];

    if ([self isViewLoaded] && ([[self view] window] == nil)) {
        self.view = nil;
        
        [self tearDownGL];
        
        if ([EAGLContext currentContext] == self.context) {
            [EAGLContext setCurrentContext:nil];
        }
        self.context = nil;
    }

    // Dispose of any resources that can be recreated.
}

- (void)setupGL
{
    [EAGLContext setCurrentContext:self.context];
    
    glEnable(GL_DEPTH_TEST);
    
    RendererImpl::Instance().SetBuiltinAssetPath( "/Assets/builtin/" );
    RendererImpl::Instance().OpenWindow( self.view.bounds.size.width, self.view.bounds.size.height, true, 1, true );
    
    texture = new TextureImpl( "/Assets/builtin/textures/checker.png", true, true, TextureImpl::Tiling::Repeat, TextureImpl::Filter::Point );
}

- (void)tearDownGL
{
    [EAGLContext setCurrentContext:self.context];
}

- (void)update
{
}

- (void)glkView:(GLKView *)view drawInRect:(CGRect)rect
{
    RendererImpl& renderer = RendererImpl::Instance();
    
    renderer.ClearScreen();
    
    renderer.Draw( texture, 284, 0, Vec4( 1, 1, 1, 1 ) );
    //renderer.Draw( &textureFromAtlas1, 78, 0, 1, 1, 1, 1 );
    
    renderer.Draw( "hello world", "aether3d_font", 1, 10, 280, Vec4( 1, 1, 1, 1 ) );
    renderer.DrawStatistics();
    
    renderer.FinishFrame();
    renderer.SwapBuffers();
}

@end
