#include <cmath>
#include <iostream>
#include "Errors.hpp"
#include "Vec3.hpp"

int main()
{
    Vec3 vec1;

    if (!vec1.IsAlmost( Vec3( 0, 0, 0 ) ))
    {
        std::cerr << "Vec3 constructor failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    vec1 = Vec3( 2, 3, 2 );
    Vec3 vec2( 3, 2, 3 );

    if (!(vec1 + vec2).IsAlmost( Vec3( 5, 5, 5 ) ))
    {
        std::cerr << "Vec3 addition failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    if (!(vec1 - vec2).IsAlmost( Vec3( -1, 1, -1 ) ))
    {
        std::cerr << "Vec3 subtract failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    if (!(vec1 * vec2).IsAlmost( Vec3( 6, 6, 6 ) ))
    {
        std::cerr << "Vec3 multiply failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    vec2 = vec2.Normalized();

    if (std::abs( vec2.Length() - 1 ) > 0.0001f)
    {
        std::cerr << "Vec3::Normalize failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }
    
    if (!(Vec3( 1, 2, 3 ) / 2.0f).IsAlmost( Vec3( 0.5f, 1.0f, 1.5f ) ))
    {
        std::cerr << "Vec3 division failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    if (!(Vec3( 1, 1, 1 ) / Vec3( 2, 2, 2 )).IsAlmost( Vec3( 0.5f, 0.5f, 0.5f ) ))
    {
        std::cerr << "Vec3 division failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }
    
    if (!(Vec3( 1, 2, 3 ) * 2.0f).IsAlmost( Vec3( 2.0f, 4.0f, 6.0f ) ))
    {
        std::cerr << "Vec3 multiply by value failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    Vec3 a( 1, 2, 3 );
    Vec3 b = a;

    if (a != b)
    {
        std::cerr << "Vec3 assignment failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    b = Vec3( 1, 1, 1 );
    Vec3 r = a + b - b;

    if (r != a)
    {
        std::cerr << "Vec3 identity failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    b = Vec3( 2, 2, 2 );
    r = a / b;

    if (!(r * b).IsAlmost( a ))
    {
        std::cerr << "Vec3 identity failed!" << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }
}

