#include <iostream>
#include "Errors.hpp"
#include "Quaternion.hpp"

static bool TestConstructor()
{
    const float tx =  1.0f;
    const float ty = -2.5f;
    const float tz =  0.4f;
    const float tw =  2.8f;

    Quaternion q( Vec3( tx, ty, tz ), tw );

    return q.x == tx && q.y == ty && q.z == tz && q.w == tw;
}

static bool TestMultiplyQ1Q2()
{
    const Quaternion q1( Vec3( 2.5f, -1.3f, -5.2f ), 1.0f );
    const Quaternion q2( Vec3( 5.4f,  2.6f,  6.7f ), 1.0f );

    const Quaternion result = q1 * q2;

    const float acceptableDelta = 0.00001f;

    if (result.w - 25.72f > acceptableDelta ||
        result.x - 12.71f > acceptableDelta ||
        result.y + 43.53f > acceptableDelta ||
        result.z - 15.02f > acceptableDelta)
    {
        return false;
    }

    return true;
}

static bool TestNormalize()
{
    Quaternion q( Vec3( 2.5f, -1.3f, -5.2f ), 1.8f );
    q.Normalize();
     // \todo test.
     return true;
}

static bool TestGetConjugate()
{
    const float tx = 1.0f;
    const float ty = -2.5f;
    const float tz = 0.4f;
    const float tw = 2.8f;

    const Quaternion q( Vec3( tx, ty, tz ), tw );
    const Quaternion result = q.Conjugate();
    const float acceptableDelta = 0.00001f;

    if (std::fabs( -result.x - tx ) > acceptableDelta ||
        std::fabs( -result.y - ty ) > acceptableDelta ||
        std::fabs( -result.z - tz ) > acceptableDelta ||
        std::fabs(  result.w - tw ) > acceptableDelta)
    {
        return false;
    }

    return true;
}

bool TestEuler()
{
    // Quaternions to Euler.
    Quaternion q;
    q.FromEuler( { 45, 0, 0 } );
    Vec3 euler = q.GetEuler();
    
    const float acceptableDelta = 0.01f;
    const bool result = 45.005f - euler.x < acceptableDelta;

    if (!result)
    {
        std::cerr << "Quaternion::GetEuler failed!" << std::endl;
    }

    return result;
}

int main()
{
    std::cout << "Testing quaternion operations." << std::endl;
    return (TestEuler() && TestConstructor() && TestGetConjugate() &&
            TestMultiplyQ1Q2() && TestNormalize()) ? (int)Error::NO_ERRORS :
                                                     (int)Error::CONTRACT_VIOLATION;
}

