// Executes tests and outputs the results to results.html.
#include <iostream>
#include <string>
#include <fstream>
#ifdef AETHER3D_UNIX
#    include <unistd.h>
#endif
#ifdef AETHER3D_WINDOWS
#    include <direct.h>
#    define getcwd _getcwd
#endif
#include <cstdlib>
#include "Aether3D.hpp"

#ifndef PATH_MAX
#    define PATH_MAX 4096
#endif
/**
   Runs tests.

   \param paramCount Command line parameter count.
   \param params Command line parameters.
   \return Test results in HTML format.
*/
std::string RunTests( int paramCount, char* params[] )
{
    const std::string testFolders[] =
    {
        "01_init",
        "02_OpenWindow",
        "03_Vectors",
        "04_Tools_common",
        "05_Quaternions",
        "06_Matrices",
        "07_Model",
        "08_SceneGraph",
        "09_MathUtils",
        "10_Shaders",
        "11_Scene"
    };

    std::string outResult = "<table border=\"1\">";
    const std::string success = " #88FF88\">Success.";
    const std::string failedMake = " #FF8888\">Didn\'t compile: ";
    const std::string failedRun = " #FF8888\">Didn\'t finish: ";
    
    char testRootFolder[ PATH_MAX ];
    getcwd( testRootFolder, PATH_MAX );
    
    for (auto& folder : testFolders)
    {
        outResult += "<tr>\n\t<td>";
        outResult += folder + "</td><td style=\"background:";

        chdir( folder.c_str() );
        const int makeResult = system( "make" );

        if (makeResult != 0)
        {
            outResult += failedMake;
            outResult += Aether3D_API::ReturnCodeToErrorString( makeResult );
            outResult += "</td></tr>\n";
            chdir( testRootFolder );
            continue;
        }

        std::string testCommand = "./test ";
        testCommand += (paramCount > 1 ? params[ 1 ] : "");
        const int runResult = system( testCommand.c_str() );

        outResult += runResult == 0 ? success : failedRun + Aether3D_API::ReturnCodeToErrorString( runResult );
        outResult += "</td></tr>\n";
        
        if (makeResult == 0)
        {
            system( "rm test" );
        }

        chdir( testRootFolder );
    }

    return outResult;
}

int main( int paramCount, char* params[] )
{
    if (paramCount != 2)
    {
#ifdef AETHER3D_UNIX
        std::cout << "Usage example: ./RunTests ../libaether3d_null.dylib" << std::endl;
#endif
#ifdef AETHER3D_WINDOWS
        std::cout << "Usage example: RunTests ../libaether3d_null.dll" << std::endl;
#endif
        return 1;
    }

    std::string header = "<html><head><title>Aether3D Unit Test Results</title></head><body><h1>Test Results ";
    header += (paramCount > 1 ? params[ 1 ] : "");
    header += " ";
    header += __DATE__;
    header += " ";
    header += __TIME__;
    header += "</h1>\n";
    std::string footer = "</body></html>";
    std::string results = RunTests( paramCount, params );
    
    std::ofstream ofs( "results.html" );
    ofs << header << results << footer;

    std::cout << "Wrote results into results.html" << std::endl;
}

